@@ -21,6 +21,7 @@
 
 const EvDocument = imports.gi.EvinceDocument;
 const EvView = imports.gi.EvinceView;
+const LOKView = imports.lokview;
 const GdkPixbuf = imports.gi.GdkPixbuf;
 const Gio = imports.gi.Gio;
 const Gd = imports.gi.Gd;
@@ -44,6 +45,13 @@ const Search = imports.search;
 const TrackerUtils = imports.trackerUtils;
 const Utils = imports.utils;
 
+// Those are the per-document-type views
+const ViewType = {
+    NONE: 0,
+    EV: 1,
+    LOK: 2
+};
+
 const DeleteItemJob = new Lang.Class({
     Name: 'DeleteItemJob',
 // deletes the given resource
@@ -233,6 +241,7 @@ const DocCommon = new Lang.Class({
         this.origPixbuf = null;
         this.defaultApp = null;
         this.defaultAppName = null;
+        this.viewType = ViewType.NONE;
 
         this.mimeType = null;
         this.rdfType = null;
@@ -591,6 +600,40 @@ const DocCommon = new Lang.Class({
         Application.searchCategoryManager.disconnect(this._filterId);
     },
 
+    loadLocal: function(passwd, cancellable, callback) {
+        if (this.mimeType == 'application/epub+zip' ||
+            this.mimeType == 'application/x-mobipocket-ebook' ||
+            this.mimeType == 'application/x-fictionbook+xml' ||
+            this.mimeType == 'application/x-zip-compressed-fb2') {
+            let exception = new GLib.Error(Gio.IOErrorEnum,
+                                           Gio.IOErrorEnum.NOT_SUPPORTED,
+                                           "Internal error: Ebooks preview isn't support yet");
+            callback(this, null, exception);
+            return;
+        }
+
+        if (LOKView.isOpenDocumentFormat(this.mimeType) && !Application.application.isBooks) {
+            let exception = null;
+            if (!LOKView.isAvailable()) {
+                exception = new GLib.Error(Gio.IOErrorEnum,
+                                           Gio.IOErrorEnum.NOT_SUPPORTED,
+                                           "Internal error: LibreOffice isn't available");
+            }
+            callback (this, null, exception);
+            return;
+        }
+
+        GdPrivate.pdf_loader_load_uri_async(this.uri, passwd, cancellable, Lang.bind(this,
+            function(source, res) {
+                try {
+                    let docModel = GdPrivate.pdf_loader_load_uri_finish(res);
+                    callback(this, docModel, null);
+                } catch (e) {
+                    callback(this, null, e);
+                }
+            }));
+    },
+
     open: function(screen, timestamp) {
         if (!this.defaultAppName)
             return;
@@ -658,6 +701,14 @@ const DocCommon = new Lang.Class({
         log('Error: DocCommon implementations must override getSourceLink');
     },
 
+    updateViewType: function() {
+        if (LOKView.isOpenDocumentFormat(this.mimeType) && !Application.application.isBooks) {
+            this.viewType = ViewType.LOK;
+        } else {
+            this.viewType = ViewType.EV;
+        }
+    },
+
     getWhere: function() {
         let retval = '';
 
@@ -665,7 +716,7 @@ const DocCommon = new Lang.Class({
             retval = '{ ?urn nie:isPartOf <' + this.id + '> }';
 
         return retval;
-    }
+    },
 });
 Signals.addSignalMethods(DocCommon.prototype);
 
@@ -681,13 +732,25 @@ const LocalDocument = new Lang.Class({
         this.sourceName = _("Local");
 
         if (this.mimeType) {
+            let defaultApp = Gio.app_info_get_default_for_type(this.mimeType, true);
+            let recommendedApp = null;
+
             let apps = Gio.app_info_get_recommended_for_type (this.mimeType);
             for (let i = 0; i < apps.length; i++) {
                 if (apps[i].supports_uris ()) {
-                    this.defaultApp = apps[i];
-                    break;
+                    if (defaultApp && apps[i].equal (defaultApp)) {
+                        // Found the recommended app that's also the default
+                        recommendedApp = apps[i];
+                        break;
+                    }
+                    // Set the first recommendedApp as the default if
+                    // they don't match
+                    if (!recommendedApp)
+                        recommendedApp = apps[i];
                 }
             }
+
+            this.defaultApp = recommendedApp;
         }
 
         if (this.defaultApp)
@@ -730,26 +793,7 @@ const LocalDocument = new Lang.Class({
     },
 
     load: function(passwd, cancellable, callback) {
-        if (this.mimeType == 'application/epub+zip' ||
-            this.mimeType == 'application/x-mobipocket-ebook' ||
-            this.mimeType == 'application/x-fictionbook+xml' ||
-            this.mimeType == 'application/x-zip-compressed-fb2') {
-            let exception = new GLib.Error(Gio.IOErrorEnum,
-                                           Gio.IOErrorEnum.NOT_SUPPORTED,
-                                           "Internal error: Ebooks preview isn't support yet");
-            callback(this, null, exception);
-            return;
-        }
-
-        GdPrivate.pdf_loader_load_uri_async(this.uri, passwd, cancellable, Lang.bind(this,
-            function(source, res) {
-                try {
-                    let docModel = GdPrivate.pdf_loader_load_uri_finish(res);
-                    callback(this, docModel, null);
-                } catch (e) {
-                    callback(this, null, e);
-                }
-            }));
+        this.loadLocal(passwd, cancellable, callback);
     },
 
     canEdit: function() {
@@ -1001,15 +1045,7 @@ const OwncloudDocument = new Lang.Class({
     },
 
     load: function(passwd, cancellable, callback) {
-        GdPrivate.pdf_loader_load_uri_async(this.uri, passwd, cancellable, Lang.bind(this,
-            function(source, res) {
-                try {
-                    let docModel = GdPrivate.pdf_loader_load_uri_finish(res);
-                    callback(this, docModel, null);
-                } catch (e) {
-                    callback(this, null, e);
-                }
-            }));
+        this.loadLocal(passwd, cancellable, callback);
     },
 
     canEdit: function() {
@@ -1313,6 +1349,8 @@ const DocumentManager = new Lang.Class({
             case Gio.IOErrorEnum.NOT_SUPPORTED:
                 if (Application.application.isBooks)
                     message = _("You are using a preview of Books. Full viewing capabilities are coming soon!");
+                else
+                    message = _("LibreOffice support is not available. Please contact your system administrator.");
                 break;
             default:
                 break;
@@ -1348,7 +1386,8 @@ const DocumentManager = new Lang.Class({
 
         // save loaded model and signal
         this._activeDocModel = docModel;
-        this._activeDocModel.set_continuous(false);
+        if (this._activeModel)
+            this._activeDocModel.set_continuous(false);
 
         // load metadata
         this._connectMetadata(docModel);
@@ -1369,6 +1408,7 @@ const DocumentManager = new Lang.Class({
         this._clearActiveDocModel();
 
         this._loaderCancellable = new Gio.Cancellable();
+        doc.updateViewType();
         this.emit('load-started', doc);
         doc.load(passwd, this._loaderCancellable, Lang.bind(this, this._onDocumentLoaded));
     },
@@ -1427,6 +1467,7 @@ const DocumentManager = new Lang.Class({
             recentManager.add_item(doc.uri);
 
             this._loaderCancellable = new Gio.Cancellable();
+            doc.updateViewType();
             this.emit('load-started', doc);
             doc.load(null, this._loaderCancellable, Lang.bind(this, this._onDocumentLoaded));
         }
@@ -1464,6 +1505,8 @@ const DocumentManager = new Lang.Class({
     },
 
     _connectMetadata: function(docModel) {
+        if (!docModel)
+            return;
         let evDoc = docModel.get_document();
         let file = Gio.File.new_for_uri(evDoc.get_uri());
         if (!GdPrivate.is_metadata_supported_for_file(file))
